//===========================================================================
// Released under Anonymity
// Feel free to embrace and extend this code!
// 11/16/02
// 
// ResourceExtractor.h
// 
// Declaration of a class, an instance of which will extract resources 
// from a large file into several smaller files
//===========================================================================

//===========================================================================
// Include guard
//===========================================================================
#ifndef __RESOURCE_EXTRACTOR_H_INCLUDED
#define __RESOURCE_EXTRACTOR_H_INCLUDED

//===========================================================================
// Other included files
//===========================================================================
#include "projectpch.h"
#include "ResourceExtractorListener.h"

//===========================================================================
// CResourceExtractor class declaration
//===========================================================================
class CResourceExtractor
{
private:
	
	class State
	{
	protected:
		CResourceExtractor* m_pOwner;
	public:
		State(CResourceExtractor* pOwningExtractor);
		virtual void processByte(int)=0;
		virtual void beginState(){}
	};
	friend class State;

	class TransientState:public State
	{
	public:
		TransientState(CResourceExtractor* pOwningExtractor);
		void processByte(int);
	};
	friend class TransientState;

	class SearchingForStart:public State
	{
		int m_compareidx;
	public:
		SearchingForStart(CResourceExtractor* pOwningExtractor);
		void beginState();
		void processByte(int);
	};
	friend class SearchingForStart;

	class CreatingFile: public TransientState
	{
	public:
		CreatingFile(CResourceExtractor* pOwningExtractor);
		void beginState();
	};
	friend class CreatingFile;

	class ExtractingFile:public State
	{
		int m_compareidx;
	public:
		ExtractingFile(CResourceExtractor* pOwningExtractor);
		void beginState();
		void processByte(int);
	};
	friend class ExtractingFile;

	class ScanningTrailingBytes :public State
	{
	public:
		ScanningTrailingBytes(CResourceExtractor* pOwningExtractor);
		void processByte(int);
	};
	friend class ScanningTrailingBytes;

	class ClosingFile : public TransientState
	{
	public:
		ClosingFile(CResourceExtractor* pOwningExtractor);
		void beginState();
	};
	friend class ClosingFile;

	void GotoState(State* pNewState);
	// call this to set the extractor into an error state
	void fatalExtractorError(const char* szErrorMsg);
	bool m_bExtractorOK;
	char m_szErrorMessage[200];
	
	FILE* m_pfile;
	FILE* m_poutfile;
	int m_numTrailingBytes;

	char* m_szOutputDirectory;
	char* m_szExtension;
	
	//memory buffers, not necessarily ASCII strings
	char* m_szStartCompareString;
	int m_startcomparelen;
	char* m_szEndCompareString;
	int m_endcomparelen;

	bool m_bDoTrailingBytes;
	unsigned char m_cTrailingByte;

	char m_szCurrFileName[200];
	fpos_t m_fp;
	fpos_t m_fpend;

	State* m_pCurrentState;

	//state overrides
	void processByte(int c);

	IResourceExtractorListener* m_output;

	int m_outfilenum;
	char* getOutFilename();
	size_t m_FilePos;
	size_t m_CurrFileSize;
	size_t m_FileSize;
	char* m_szMainFileName;
	
public:
	CResourceExtractor(char* filename, char* extension, char* outputDirectory, char* startSentinel, int startSentinelLen,char* endSentinel, int endSentinelLen,bool dotrail,unsigned char trailbyte,IResourceExtractorListener* pOutput);
	
	bool isOK(){return m_bExtractorOK;}
	const char* getErrorMessage(){return m_szErrorMessage;}
	
	void go();
	~CResourceExtractor();
	// backTracks a number of characters in the filestream
	void backTrack(int numChars);
};
#endif