//===========================================================================
// Released under Anonymity
// Feel free to embrace and extend this code!
// 11/16/02
// 
// ResourceExtractorState_TransientState.cpp
// 
// Implementation of a base class for all transient states used by the
// resource extractor
//===========================================================================

//===========================================================================
// Included Files
//===========================================================================
#include "projectpch.h"
#include "ResourceExtractor.h"

//===========================================================================
// Definition of Constructor: CResourceExtractor::TransientState::
//                TransientState(CResourceExtractor* pOwningExtractor)       
//
// This constructor simply defers to the super class constructor
//===========================================================================
CResourceExtractor::TransientState::TransientState(CResourceExtractor* pOwningExtractor)
:State(pOwningExtractor)
{
}

//===========================================================================
// Definition of Method: CResourceExtractor::TransientState::
//                                                     processByte(int c)       
//
// given that we're in a transient state, no bytes should ever be processed
// thus this method asserts with a helpful error message.
//
// trick: !<string literal> casts the const char* of the literal to a void*
// and the ! casts it to a bool and evaluates the complement.  This is always
// false because the address of the string literal is always non-null
// since asserts print the expression in their parens, this is a concise way
// to display an error message to a developer doing any debugging.
//===========================================================================
void CResourceExtractor::TransientState::processByte(int c)
{
	//transient states never process bytes.
	assert(!"NO BYTES SHOULD BE PROCESSED IN A TRANSIENT STATE!");
}