//===========================================================================
// Released under Anonymity
// Feel free to embrace and extend this code!
// 11/16/02
// 
// main.cpp 
// 
// The driver file of this project
//===========================================================================

//===========================================================================
// Included Files
//===========================================================================
#include "projectpch.h"
#include "ResourceExtractorStatusOutput.h"
#include "ResourceExtractor.h"

//===========================================================================
// Defines and Global Data
//===========================================================================
#define RFIS_FILE_TOO_LONG 1
#define RFIS_NO_FILE 2
#define RFIS_SUCCESS 3

#define INPUT_FILE_SIZE 256
char g_szOutputDirectory[_MAX_PATH]="oggs";
char g_szOutputExtension[_MAX_EXT]="ogg";
char g_szStartCompareString[INPUT_FILE_SIZE]="OggS";
int g_startcomparelen=4;
char g_szEndCompareString[INPUT_FILE_SIZE]={0x00,0x00, 0x00,0x00, 0x00,0x00, 0x00,0x00, 0x00,0x00, 0x00,0x00, 0x00,0x00, 0x00,0x00,0x00,0x00, 0x00,0x00, 0x00,0x00, 0x00,0x00, 0x00,0x00, 0x00,0x00, 0x00,0x00, 0x00,0x00};
int g_endcomparelen=32;
bool g_bDoTrailingBytes=true;
unsigned char g_cTrailingByte=0;

//===========================================================================
// Definition of Function: readFileIntoString(char* filename,char* string,
//                                         int maxbytes,int* outNumBytesRead)
// 
// readFileIntoString takes a filename, a character buffer, the size of said
// buffer and a pointer to an integer output paramter representing the 
// number of bytes read out of the file and into the buffer. (out condition:
// outNumBytesRead will always be less than or equal to maxbytes).  
//
// Returns: one of the 3 RFIS_ return codes above, which are self explanitory.
//===========================================================================
int readFileIntoString(char* filename,char* string,int maxbytes,int* outNumBytesRead)
{
	FILE* f=fopen(filename,"rb");
	if (!f)
		return RFIS_NO_FILE;
	fpos_t fp;
	fpos_t fpend;
	fgetpos(f,&fp);
	fseek(f,0,SEEK_END);
	fgetpos(f,&fpend);
	fseek(f,0,SEEK_SET);
	if (fpend-fp > maxbytes)
		return RFIS_FILE_TOO_LONG;

	while (!feof(f))
	{
		*string++=fgetc(f);
		if (outNumBytesRead)
			*outNumBytesRead++;
	}
	
	return RFIS_SUCCESS;
}

//===========================================================================
// Definition of Function: startsWith(char* bigstr, char* smallstr)
// 
// startsWith determines if bigstr's first n characters match the characters
// of smallstring (where n is the length of smallstring)
//
// Returns: NULL if bigstr does NOT start with smallstr, otherwise
// the pointer to the character in bigstr occuring after the character
// matching smallstr's last character
//===========================================================================
char* startsWith(char* bigstr, char* smallstr)
{
	if (strnicmp(bigstr,smallstr,strlen(smallstr))==0)
		return bigstr+strlen(smallstr);
	return NULL;
}

//===========================================================================
// Definition of Procedure: printUsage()
// 
// printUsage prints instructions on how to use the various commandline 
// options that the program can recognize.
//===========================================================================
void printUsage()
{
	char randchar;
	time_t t=time(&t);
	srand((unsigned int)t);
	float r=((float)rand())/((float)RAND_MAX);
	randchar='a'+(int)(r*26.f);
	printf("Usage: ProjectX.exe <big file>\n");
	printf("\t\t[/out:directory in which to put extracted files]\n");
	printf("\t\t[/ext:extension for extracted files]\n");
	printf("\t\t[/start:file with start data] \n");
	printf("\t\t[/end:file with end data]\n");
	printf("\t\t[/trail:<none | character | value of trailing byte>]\n");
	printf("\n");
	printf("\tDefaults are all geared towards extracting ogg files from\n\tbigfile.dat in SoulReaver2\n");
	printf("\n");
	printf("\tExamples of trail switch:\n");
	printf("\t\t/trail:none will turn off the trailing byte scanning\n");
	printf("\t\t/trail:%c will trail the value 0x%x (the value of %c)\n",randchar,(unsigned int)randchar,randchar);
	
	r=((float)rand())/((float)RAND_MAX);
	randchar='a'+(int)(r*26.f);
	printf("\t\t/trail:0x%x specifies the trailing byte '%c' in hex\n",(unsigned int)randchar,randchar);
	
	r=((float)rand())/((float)RAND_MAX);
	randchar='a'+(int)(r*26.f);
	printf("\t\t/trail:%li specifies the trailing byte '%c' in decimal\n",(unsigned int)randchar,randchar);
	
}

//===========================================================================
// Definition of Function: main(int argc, char* argv[])
// 
// the main entrypoint of the executable. this function parses the command
// line, sets up the variables to pass to the ResourceExtractor and then
// instantiates the extractor, connecting a listener object to it
// It then tells it to go()!
//
// returns: -1 if the program fails, 0 if it is successful.
//===========================================================================
int main(int argc, char* argv[])
{
	printf("\n---\n");
	if (argc < 2)
	{
		printUsage();
		return -1;
	}
	else 
	{
		if (stricmp(argv[1],"/help")==0 || argv[1][0]=='/')
		{
			printUsage();
			return -1;
		}
		else
		{
			for (int i=2;i<argc;i++)
			{
				char* cs;
				if ((cs=startsWith(argv[i],"/out:"))!=NULL)
				{
					strcpy(g_szOutputDirectory,cs);
					if (strlen(g_szOutputDirectory) < 1)
					{
						printf("invalid output directory\n");
						return -1;
					}

				}
				else if ((cs=startsWith(argv[i],"/ext:"))!=NULL)
				{
					strcpy(g_szOutputExtension,cs);
					if (strlen(g_szOutputExtension) < 1)
					{
						printf("invalid output extension\n");
						return -1;
					}
				}
				else if ((cs=startsWith(argv[i],"/start:"))!=NULL)
				{
					int result=readFileIntoString(cs,g_szStartCompareString,INPUT_FILE_SIZE,&g_startcomparelen);
					switch(result)
					{
					case RFIS_FILE_TOO_LONG:
						printf("input file %s must be less than %li bytes\n",cs,INPUT_FILE_SIZE);
						return -1;
					case RFIS_NO_FILE:
						printf("input file %s not found\n",cs);
						return -1;
					case RFIS_SUCCESS:
						break;
					default:
						printf("unknown error!\n");
						return -1;
					}
				}
				else if ((cs=startsWith(argv[i],"/end:"))!=NULL)
				{
					int result=readFileIntoString(cs,g_szEndCompareString,INPUT_FILE_SIZE,&g_endcomparelen);
					switch(result)
					{
					case RFIS_FILE_TOO_LONG:
						printf("input file %s must be less than %li bytes\n",cs,INPUT_FILE_SIZE);
						return -1;
					case RFIS_NO_FILE:
						printf("input file %s not found\n",cs);
						return -1;
					case RFIS_SUCCESS:
						break;
					default:
						printf("unknown error!\n");
						return -1;
					}
				}
				else if ((cs=startsWith(argv[i],"/trail:"))!=NULL)
				{
					if (stricmp(cs,"none")==0)
						g_bDoTrailingBytes=false;
					else if (strlen(cs)==1)
						g_cTrailingByte=*cs;
					//if it's a value, convert it 
					else 
					{
						char* trailer=NULL;
						long l=strtol(cs,&trailer,0);
						if (l > 256 || (trailer && trailer[0]!=0))
						{
							printf("invalid trailing byte value, %l is over 0xFF\n",l);
							return -1;
						}
						else
							g_cTrailingByte=(unsigned char)(l&0xFF);
					}
					
				}
				else
				{
					printUsage();
					return -1;
				}
			}
		}
	}

	//instantiate the status outputter
	CResourceExtractorStatusOutput StatusOutput;

	//construct the extractor
	CResourceExtractor extractor
	(
		argv[1],
		g_szOutputExtension,
		g_szOutputDirectory,
		g_szStartCompareString,
		g_startcomparelen,
		g_szEndCompareString,
		g_endcomparelen,
		g_bDoTrailingBytes,
		g_cTrailingByte,
		&StatusOutput
	);
	
	if (!extractor.isOK())
	{
		printf("Error: %s\n",extractor.getErrorMessage());
		return -1;
	}

	//and it's off!
	extractor.go();
	if (!extractor.isOK())
	{
		printf("Error: %s\n",extractor.getErrorMessage());
		return -1;
	}
	return 0;
}